package com.air.applets.controller;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.air.applets.dto.MeasureStatisticsDto;
import com.air.applets.dto.MeasureSubmitDto;
import com.air.applets.dto.MeasureTrendDto;
import com.air.applets.entity.MeasureData;
import com.air.applets.service.MeasureDataService;
import com.air.applets.vo.MeasureDataVo;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cinderella.framework.common.core.util.R;
import com.cinderella.framework.common.security.util.SecurityUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 小程序测算数据管理
 *
 * @author peihao
 * @date 2021-05-18 09:40:33
 */
@RestController
@AllArgsConstructor
@RequestMapping("/applets/measuredata")
@Api(value = "measuredata", tags = "小程序测算数据管理")
public class MeasureDataController {

    private final MeasureDataService measureDataService;

    /**
     * 测算记录列表
     *
     * @param page        分页对象
     * @param measureData 测算数据
     * @return
     */
    @ApiOperation(value = "测算记录列表分页查询", notes = "测算记录列表分页查询")
    @GetMapping("/page")
    public R<IPage<MeasureData>> getMeasureDataPage(Page page, MeasureData measureData) {
        return R.ok(measureDataService.getMeasureDataPage(page, measureData));
    }

    /**
     * 通过id查询测算数据
     *
     * @param measureDataId id
     * @return R
     */
    @ApiOperation(value = "通过id查询测算详情", notes = "通过id查询测算详情")
    @GetMapping("/{measureDataId}")
    public R<MeasureData> getById(@PathVariable("measureDataId") Long measureDataId) {
        return R.ok(measureDataService.getById(measureDataId));
    }

    /**
     * 通过地块id查询测算数据
     *
     * @param landListedId
     * @return R
     */
    @ApiOperation(value = "通过地块id查询测算详情", notes = "通过地块id查询测算详情")
    @GetMapping("/getByLandListedId/{landListedId}")
    public R<MeasureData> getByLandListedId(@PathVariable("landListedId") String landListedId) {
        return R.ok(measureDataService.getOne(Wrappers.<MeasureData>query().lambda().eq(MeasureData::getLandListedId, landListedId)));
    }

    /**
     * 保存测算数据
     *
     * @param measureDataVo 测算数据
     * @return R
     */
    @ApiOperation(value = "保存测算数据", notes = "保存测算数据")
    @PostMapping
    public R<Boolean> save(@RequestBody @Validated MeasureDataVo measureDataVo) {
        if (StrUtil.isEmpty(measureDataVo.getLandListedId()) && StrUtil.isEmpty(measureDataVo.getProposedseriaId())) {
            return R.failed("地块id不能为空");
        }
        if (StrUtil.isNotEmpty(measureDataVo.getLandListedId()) && StrUtil.isNotEmpty(measureDataVo.getProposedseriaId())) {
            return R.failed("已挂牌地块标识和拟挂牌地块标识选填其一！");
        }
        MeasureData measureData;
        if (null == measureDataVo.getMeasureDataId()) {
            if (null == measureDataVo.getLandListedId()) {
                measureData = measureDataService.getOne(Wrappers.<MeasureData>query().lambda()
                        .eq(MeasureData::getProposedseriaId, measureDataVo.getProposedseriaId()));
            } else {
                measureData = measureDataService.getOne(Wrappers.<MeasureData>query().lambda()
                        .eq(MeasureData::getLandListedId, measureDataVo.getLandListedId()));
            }
            if (null != measureData) {
                return R.failed("该地块已存在测算数据，不能再次新增");
            }
        }else {
            measureData = measureDataService.getById(measureDataVo.getMeasureDataId());
            if (measureData == null){
                return R.failed("测算数据id不正确");
            }
            if ("1100".equals(measureData.getStatusCd())){
                return R.failed("测算数据提交后无法在修改！");
            }
            //如果修改钱为高级测算数据，则不能改为基础测算数据
            if (1 == measureData.getMeasureType() && 0 == measureDataVo.getMeasureType()){
                return R.failed("高级测算数据不能改为基础测算数据");
            }
        }
        MeasureData measure = new MeasureData();
        BeanUtil.copyProperties(measureDataVo, measure);
        measure.setCreateUserId(SecurityUtils.getUser().getId().toString());
        return R.ok(measureDataService.saveOrUpdate(measure));
    }

    /**
     * 提交测算数据
     *
     * @param measureDataVo 测算数据
     * @return R
     */
    @ApiOperation(value = "提交测算数据", notes = "提交测算数据")
    @PostMapping("submit")
    public R<MeasureSubmitDto> submit(@RequestBody MeasureDataVo measureDataVo) {
        return R.ok(new MeasureSubmitDto());
    }

    /**
     * 查询用户测算结果统计
     *
     * @return R
     */
    @ApiOperation(value = "测算结果统计", notes = "测算结果统计")
    @GetMapping("/statistics")
    public R<MeasureStatisticsDto> statistics() {
        return R.ok(measureDataService.statistics());
    }

    /**
     * 查询用户测算趋势统计
     *
     * @return R
     */
    @ApiOperation(value = "测算趋势统计", notes = "测算趋势统计")
    @GetMapping("/trend")
    public R<List<MeasureTrendDto>> trend() {
        return R.ok(measureDataService.trend());
    }

    /**
     * 通过id删除测算数据
     *
     * @param measureDataId 测算数据id
     * @return R
     */
    @ApiOperation(value = "通过id删除测算数据", notes = "通过id删除测算数据")
    @DeleteMapping("/{measureDataId}")
    public R<Boolean> removeById(@PathVariable Long measureDataId) {
        return R.ok(measureDataService.removeById(measureDataId));
    }
}
