package com.air.land.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.air.applets.dto.LandDto;
import com.air.applets.vo.LandVo;
import com.air.common.Constant;
import com.air.enums.LandListStatusEnum;
import com.air.land.entity.*;
import com.air.land.mapper.LandListedMapper;
import com.air.land.mapper.LandToListMapper;
import com.air.land.service.*;
import com.air.utils.DateUtil;
import com.air.utils.ExcelUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.cinderella.framework.common.core.exception.BusinessException;
import com.cinderella.framework.common.core.util.R;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * 拟挂牌地块，全部是用户手动管理的数据。
 *
 * @author peihao
 * @date 2021-05-18 09:40:09
 */
@Slf4j
@Service
@AllArgsConstructor
public class LandToListServiceImpl extends ServiceImpl<LandToListMapper, LandToList> implements LandToListService {

    private LandListedMapper landListedMapper;

    private LandToListLonLatService landToListLonLatService;

    private LandToListConductEnterpriseService conductEnterpriseService;

    private LandToListConstructionPlanService constructionPlanService;

    private LandToListAttachmentService attachmentService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public R<Boolean> sendLandList(String proposedseriaId) {
        LandToList landToList = this.getById(proposedseriaId);
        if (null == landToList) {
            return R.failed("未查询到数据");
        }
        LandListed landListed = new LandListed();
        BeanUtil.copyProperties(landToList, landListed);

        landListedMapper.insert(landListed);
        LandToList land = new LandToList();
        land.setProposedseriaId(proposedseriaId);
        land.setStatusCd(Constant.LAND_TO_STATUS_1100);
        return R.ok(this.updateById(land), "发送成功");
    }

    @Override
    public List<List<String>> getLandList(List<String> ids) {
        List<LandToList> landListeds = this.listByIds(ids);
        if (CollectionUtil.isEmpty(landListeds)) {
            throw new BusinessException("未查询到拟挂牌地块数据！");
        }
        List<List<String>> list = new ArrayList<>();
        landListeds.forEach(landListed -> {
            list.add(entityToList(landListed));
        });
        return list;
    }

    @Override
    public boolean importLandToListExcel(MultipartFile file) {
        List<LandToList> dataList;
        try {
            dataList = ExcelUtil.importExcel(file.getInputStream(), 0, 1, 0, LandToList.class, "config/landToList-config.json");
        } catch (Exception e) {
            log.error("导入拟挂牌地块数据解析出错", e);
            throw new BusinessException("导入拟挂牌地块数据出错");
        }
        if (CollectionUtil.isNotEmpty(dataList)) {
            dataList.forEach(landToList -> {
                landToList.setProposedseriaId(DateUtil.landToListIdfromDate(LocalDateTime.now(), "yyyyMMdd", this.selectId()));
                //土地出让面积，计算土地出让面积（亩）（出让面积/2000*3）
                BigDecimal landTransferSquare = landToList.getLandTransferSquare();
                if (landTransferSquare != null) {
                    landToList.setLandTransferMu(landTransferSquare.divide(new BigDecimal(2000), 4, BigDecimal.ROUND_HALF_UP)
                            .multiply(new BigDecimal(3)).setScale(0, BigDecimal.ROUND_HALF_UP));
                }
                //总建筑面积
                BigDecimal totalConsArea = landToList.getTotalConsArea();
                if (totalConsArea != null) {
                    //计算容积率(总建筑面积/土地出让面积)
                    if (landTransferSquare != null) {
                        landToList.setPlotRatio(totalConsArea.doubleValue() == 0 ? new BigDecimal(0) : totalConsArea.divide(landTransferSquare, 2, BigDecimal.ROUND_HALF_UP));
                    }
                    //出让价款起始价（万元），计算起始楼面价(元/㎡) （出让价款起始价/总建筑面积*10000）
                    BigDecimal transferPrice = landToList.getTransferPrice();
                    if (transferPrice != null) {
                        landToList.setStartingFloorPrice((transferPrice.doubleValue() == 0 || totalConsArea.doubleValue() == 0)
                                ? new BigDecimal(0) : transferPrice.divide(totalConsArea, 4, BigDecimal.ROUND_HALF_UP)
                                .multiply(new BigDecimal(10000)).setScale(0, BigDecimal.ROUND_HALF_UP));
                    }
                    //商业面积，商务面积
                    BigDecimal bizSpace = landToList.getBizSpace() == null ? new BigDecimal(0) : landToList.getBizSpace();
                    BigDecimal commerceSpace = landToList.getCommerceSpace() == null ? new BigDecimal(0) : landToList.getCommerceSpace();
                    //计算商业商务占比（（商业面积+商务面积）/总建筑面积）
                    landToList.setBizCommerceRate(totalConsArea.doubleValue() == 0 ? new BigDecimal(0) : (bizSpace.add(commerceSpace))
                            .divide(totalConsArea, 4, BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_HALF_UP));
                }
            });
        }
        return this.saveBatch(dataList);
    }

    @Override
    public List<LandDto> appletsLandToList(Page page, LandVo landVo) {
        List<LandDto> landDtos = baseMapper.appletsLandToList(page, landVo);
        landDtos.forEach(landDto -> {
            landDto.setIsToBeListed("1");
            landDto.setTransactionStatus(LandListStatusEnum.TO_BE_LISTED.getCode());
        });
        return landDtos;
    }

    @Override
    public Integer selectId() {
        return baseMapper.selectId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean removeLandToListByIds(List<String> proposedseriaIds) {
        this.removeByIds(proposedseriaIds);
        landToListLonLatService.remove(Wrappers.<LandToListLonLat>query().lambda().in(LandToListLonLat::getProposedseriaId, proposedseriaIds));
        conductEnterpriseService.remove(Wrappers.<LandToListConductEnterprise>query().lambda()
                .in(LandToListConductEnterprise::getProposedseriaId, proposedseriaIds));
        constructionPlanService.remove(Wrappers.<LandToListConstructionPlan>query().lambda()
                .in(LandToListConstructionPlan::getProposedseriaId, proposedseriaIds));
        attachmentService.remove(Wrappers.<LandToListAttachment>query().lambda().in(LandToListAttachment::getProposedseriaId, proposedseriaIds));
        return true;
    }

    /**
     * 导入拟挂牌地块数据
     *
     * @param file 导入文件
     * @return
     * @author peihao
     * @date 2021/6/17
     **/
    public List<LandToList> importLandListedExcel(MultipartFile file) throws IOException {
        // 读取excel模板
        XSSFWorkbook wb = new XSSFWorkbook(file.getInputStream());
        XSSFSheet sheet = wb.getSheetAt(0);
        int rowSum = sheet.getPhysicalNumberOfRows();
        //获取数据
        List<LandToList> result = new ArrayList<>();
        for (int i = 2; i < rowSum; i++) {
            XSSFRow sheetRow = sheet.getRow(i);
            LandToList landToList = new LandToList();
            landToList.setProposedseriaId(DateUtil.landToListIdfromDate(LocalDateTime.now(), "yyyyMMdd", this.selectId()));
            //拟挂牌信息
            landToList.setSupplyStatus(getCellStrValue(sheetRow.getCell(0)));
            landToList.setEstimatedListingTime(getCellStrValue(sheetRow.getCell(1)));
            //公告信息
            landToList.setAnnoId(getCellValue(sheetRow.getCell(2)));
            log.info("公告序号为： {}", landToList.getAnnoId());
            landToList.setLandCode(getCellStrValue(sheetRow.getCell(3)));
            landToList.setLandPosition(getCellStrValue(sheetRow.getCell(4)));
            landToList.setLandUsage(getCellStrValue(sheetRow.getCell(5)));
            landToList.setLandTransferSquare(numberFormat(getCellValue(sheetRow.getCell(6)), 0));
            landToList.setLandTransferMu(numberFormat(getCellValue(sheetRow.getCell(7)), 0));
            landToList.setTotalConsArea(numberFormat(getCellValue(sheetRow.getCell(8)), 0));
            landToList.setPlotRatio(numberFormat(getCellValue(sheetRow.getCell(9)), 2));
            landToList.setBuildingDensity(numberFormat(getCellStrValue(sheetRow.getCell(10)), 100, 0));
            landToList.setGreenSpaceRatio(numberFormat(getCellStrValue(sheetRow.getCell(11)), 100, 0));
            landToList.setHeighPermitted(getCellStrValue(sheetRow.getCell(12)));
            landToList.setRemiseYears(getCellStrValue(sheetRow.getCell(13)));
            landToList.setTransferPrice(numberFormat(getCellStrValue(sheetRow.getCell(14)), 0));
            landToList.setStartingFloorPrice(numberFormat(getCellValue(sheetRow.getCell(15)), 0));
            landToList.setBidMargin(numberFormat(getCellStrValue(sheetRow.getCell(16)), 0));
            landToList.setBizRemark(getCellStrValue(sheetRow.getCell(17)));
            try {
                landToList.setAnnoDate(DateUtil.dateToLocalDate(sheetRow.getCell(18).getDateCellValue()));
            } catch (Exception e) {
                log.error("公告开始日期转换出错", e);
            }
            try {
                landToList.setAnnoEndDate(DateUtil.dateToLocalDate(sheetRow.getCell(19).getDateCellValue()));
            } catch (Exception e) {
                log.error("公告截止日期转换出错", e);
            }
            try {
                landToList.setAuctionDate(DateUtil.dateToLocalDate(sheetRow.getCell(20).getDateCellValue()));
            } catch (Exception e) {
                log.error("拍卖日期转换出错", e);
            }
            landToList.setLicensedArea(numberFormat(getCellStrValue(sheetRow.getCell(21)), 0));
            //规划条件
            landToList.setBizSpace(numberFormat(getCellStrValue(sheetRow.getCell(22)), 0));
            landToList.setCommerceSpace(numberFormat(getCellStrValue(sheetRow.getCell(23)), 0));
            landToList.setBizCommerceRate(numberFormat(getCellValue(sheetRow.getCell(24)), 100, 2));
            landToList.setAllocationCapacity(getCellStrValue(sheetRow.getCell(25)));
            landToList.setAllocationNotCapacity(getCellStrValue(sheetRow.getCell(26)));
            //建设条件
            landToList.setGeologicHazard(getCellStrValue(sheetRow.getCell(27)));
            landToList.setCivilAirDefence(numberFormat(getCellStrValue(sheetRow.getCell(28)), 0));
            landToList.setSpecialPlan(getCellStrValue(sheetRow.getCell(29)));
            landToList.setAssemblyBuilding(getCellStrValue(sheetRow.getCell(30)));
            landToList.setGreenBuilding(getCellStrValue(sheetRow.getCell(31)));
            landToList.setOtherConditions(getCellStrValue(sheetRow.getCell(32)));
            landToList.setPercentFinishedHousing(numberFormat(getCellStrValue(sheetRow.getCell(33)), 0));
            //竞买条件
            landToList.setBiddingConditions(getCellStrValue(sheetRow.getCell(34)));
            landToList.setHookForm(getCellStrValue(sheetRow.getCell(35)));
            landToList.setFirstGovernance(getCellStrValue(sheetRow.getCell(36)));
            landToList.setLandEnterprises(getCellStrValue(sheetRow.getCell(37)));
            landToList.setPaymentRhythm(getCellStrValue(sheetRow.getCell(38)));
            landToList.setBargainor(getCellStrValue(sheetRow.getCell(39)));
            //地理信息
            landToList.setCity(getCellStrValue(sheetRow.getCell(40)));
            landToList.setCanton(getCellStrValue(sheetRow.getCell(41)));
            landToList.setSpecialZone(getCellStrValue(sheetRow.getCell(42)));
            landToList.setLoopWire(getCellStrValue(sheetRow.getCell(43)));
            landToList.setBigGroup(getCellStrValue(sheetRow.getCell(44)));
            landToList.setSmallGroup(getCellStrValue(sheetRow.getCell(45)));
            landToList.setOvePosition(getCellStrValue(sheetRow.getCell(46)));
            landToList.setLonLatBd(getCellStrValue(sheetRow.getCell(47)));
            result.add(landToList);
        }
        return result;
    }

    /**
     * 将拟挂牌地块信息转换为excel导出格式
     *
     * @param data 拟挂牌地块信息
     * @return
     * @author peihao
     * @date 2021/8/25
     **/
    private List<String> entityToList(LandToList data) {
        List<String> list = new ArrayList<>();
        //拟挂牌信息
        list.add(data.getSupplyStatus());
        list.add(data.getEstimatedListingTime());
        //公告信息
        list.add(data.getAnnoId());
        list.add(data.getLandCode());
        list.add(data.getLandPosition());
        list.add(data.getLandUsage());
        list.add(stringFormat(data.getLandTransferSquare(), 0));
        list.add(stringFormat(data.getLandTransferMu(), 0));
        list.add(stringFormat(data.getTotalConsArea(), 0));
        list.add(stringFormat(data.getPlotRatio(), 2));
        list.add(strRateFormat(data.getBuildingDensity(), 0));
        list.add(strRateFormat(data.getGreenSpaceRatio(), 0));
        list.add(data.getHeighPermitted());
        list.add(data.getRemiseYears());
        list.add(stringFormat(data.getTransferPrice(), 0));
        list.add(stringFormat(data.getStartingFloorPrice(), 0));
        list.add(stringFormat(data.getBidMargin(), 0));
        list.add(data.getBizRemark());
        list.add(DateUtil.fromString(data.getAnnoDate(), "yyyy-MM-dd"));
        list.add(DateUtil.fromString(data.getAnnoEndDate(), "yyyy-MM-dd"));
        list.add(DateUtil.fromString(data.getAuctionDate(), "yyyy-MM-dd"));
        list.add(stringFormat(data.getLicensedArea(), 0));
        //规划条件
        list.add(stringFormat(data.getBizSpace(), 0));
        list.add(stringFormat(data.getCommerceSpace(), 0));
        list.add(strRateFormat(data.getBizCommerceRate(), 0));
        list.add(data.getAllocationCapacity());
        list.add(data.getAllocationNotCapacity());
        //建设条件
        list.add(data.getGeologicHazard());
        list.add(stringFormat(data.getCivilAirDefence(), 0));
        list.add(data.getSpecialPlan());
        list.add(data.getAssemblyBuilding());
        list.add(data.getGreenBuilding());
        list.add(data.getOtherConditions());
        list.add(stringFormat(data.getPercentFinishedHousing(), 0));
        //竞买条件
        list.add(data.getBiddingConditions());
        list.add(data.getHookForm());
        list.add(data.getFirstGovernance());
        list.add(data.getLandEnterprises());
        list.add(data.getPaymentRhythm());
        list.add(data.getBargainor());
        //地理信息
        list.add(data.getCity());
        list.add(data.getCanton());
        list.add(data.getSpecialZone());
        list.add(data.getLoopWire());
        list.add(data.getBigGroup());
        list.add(data.getSmallGroup());
        list.add(data.getOvePosition());
        list.add(data.getLonLatBd());
        return list;
    }

    /**
     * 获取有计算公式列的值
     *
     * @param cell
     * @return
     * @author peihao
     * @date 2021/6/25
     **/
    private String getCellValue(XSSFCell cell) {
        String value;
        try {
            value = cell.getRawValue();
        } catch (IllegalStateException e) {
            value = cell.toString();
        }
        return valueFormat(value);
    }

    /**
     * 获取没有计算公式列的值
     *
     * @param cell
     * @return
     * @author peihao
     * @date 2021/6/25
     **/
    private String getCellStrValue(XSSFCell cell) {
        String value = null;
        try {
            value = cell.toString();
        } catch (IllegalStateException e) {
            e.printStackTrace();
        }
        return valueFormat(value);
    }

    /**
     * 格式化列的值
     *
     * @param value
     * @return
     * @author peihao
     * @date 2021/6/25
     **/
    private String valueFormat(String value) {
        value = StringUtils.isNotEmpty(value) ? value.trim() : value;
        value = StringUtils.equals("-", value) ? "" : value;
        return value;
    }

    /**
     * 格式化列值为数字
     *
     * @param decimal 保留小数位
     * @param value   数据
     * @return
     * @author peihao
     * @date 2021/6/25
     **/
    private BigDecimal numberFormat(String value, int decimal) {
        if (StringUtils.isEmpty(value)) {
            return null;
        }
        return new BigDecimal(value).setScale(decimal, BigDecimal.ROUND_HALF_UP);
    }

    private BigDecimal numberFormat(String value, int mulitple, int decimal) {
        if (StringUtils.isEmpty(value)) {
            return null;
        }
        return new BigDecimal(value).multiply(new BigDecimal(mulitple)).setScale(decimal, BigDecimal.ROUND_HALF_UP);
    }

    /**
     * 格式化列值为字符串
     *
     * @param value   数据
     * @param decimal 保留小数位
     * @return
     * @author peihao
     * @date 2021/6/25
     **/
    private String stringFormat(BigDecimal value, int decimal) {
        if (value == null) {
            return "";
        }
        return value.setScale(decimal, BigDecimal.ROUND_HALF_UP).toString();
    }

    /**
     * 数字转换百分比
     *
     * @return
     * @author peihao
     * @date 2021/8/25
     **/
    private String strRateFormat(BigDecimal value, int decimal) {
        return value == null ? null : stringFormat(value, decimal) + "%";
    }
}
